#import <machkit/NXData.h>

@protocol Interrupt
-(oneway void)interrupt;
@end

@protocol BaseValueGetting
-(int)numSubValues;
-(id <BaseValueGetting>)subValueAt: (int)nValue;
-(id <BaseValueGetting>)subValueAt: (int)nValue
           withFlags: (out unsigned *)pFlags
	     andName: (out char **)pName;
-(id <BaseValueGetting>)subValueNamed: (in char *)valueName index:(out int *)index;
-(id <BaseValueGetting>)subValueFor: (id <BaseValueGetting>)value from: (int *)from index: (int *)index;
@end

@protocol Value <BaseValueGetting>

-(char *)valueName;
-(unsigned)size;
-(unsigned)scalerValueFrom: (unsigned)from;
-(float)floatValueFrom: (unsigned)from;
-(char *)enumValueFrom: (unsigned)from;
-(char *)stringValueFrom: (unsigned)from;
-(char *)functionValueFrom: (unsigned)from;
-(unsigned)addressFrom: (unsigned)from;
-(unsigned)addressFrom: (unsigned)from isValid: (out int *)isValid;
-(unsigned)flags;
-(char *)typeName;
-(int)castToTypeNamed: (in char *)typeName from: (unsigned)from;

@end

@protocol Frame <BaseValueGetting>
-(int)hasSymbols;
-(char *)functionName;
-(char *)functionNameAndHasSymbols: (int *)hasSymbols;
-(unsigned)frame;
-(unsigned)pc;
-(char *)fileNameAndLine: (unsigned *)line;
-(int)frameNumber;
@end

@protocol Breakpoint
-(char *)fileName;
-(int)line;
-(BOOL)enabled;
-(char *)commands;
-(char *)condition;
-(char *)expression;
-(char *)function;
@end

@protocol GdbController
-(oneway void)breakInFile: (in char *)filename atLine: (int)line;
-(oneway void)untilInFile: (in char *)filename atLine: (int)line;
-(oneway void)step;
-(oneway void)next;
-(oneway void)continue;
-(oneway void)up;
-(oneway void)frame;
-(oneway void)down;
-(oneway void)finish;
-(oneway void)run;
-(oneway void)quit;
-(oneway void)printExpression: (in char *)selection;
-(oneway void)printExpressionIndirect: (in char *)selection;
-(oneway void)selectFrame: (int)frame;
-(oneway void)interrupt;
-(int)getFramesUpTo: (int)maxFrames selectedFrame: (out int *)sel symbolsOnly: (BOOL)sOnly;
-(id <Frame>)frameAt: (int)frameIndex
     functionName: (out char **)pFunctionName
             from: (out unsigned int *)pFrom;
-(id <Frame>)frameAt: (int)frameIndex
             from: (out unsigned int *)pFrom;
-(id <Frame>)frameAt: (int)frameIndex;
-(char *)functionNameAt: (int)frameIndex;
-(char *)functionNameAt: (int)frameIndex hasSymbols: (int *)hasSymbols;
-(oneway void)errorOpeningFile: (char *)filename
                      fromLine: (int)start to: (int)end;
-(oneway void)executeCommand: (char *)command;
-(oneway void)setClient: anObject;
-(oneway void)addSourceDirectory: (const char *)directory;
-(int)numBreakpoints;
-(id <Breakpoint>)breakpointAt: (int)index;
-(NXData *)completionsForPrefix: (const char *)prefix;
@end

@protocol GdbUser
-(oneway void)openFile: (char *)fileName onHost: (char *)hostName atTrueLine: (int)line;
-(oneway void)openFile: (char *)fileName onHost: (char *)hostName fromTrueLine: (int)startLine to: (int)endLine;
-(oneway void)setInterruptHandler: handler;
-(oneway void)disconnect;
-(oneway void)frameChanged;
-(oneway void)programIsRunning;
-(oneway void)programStopped;
-(oneway void)programIsDead;
-(oneway void)setController: (id <GdbController>)anObject;
-(int)isNewUser;
-(oneway void)setGdbName: (const char *)theName;
@end

@protocol GdbNewUser
-(oneway void)openFile: (char *)fileName onHost: (char *)hostName atTrueLine: (int)line;
-(oneway void)openFile: (char *)fileName onHost: (char *)hostName fromTrueLine: (int)startLine to: (int)endLine;
-(oneway void)setInterruptHandler: handler;
-(oneway void)disconnect;
-(oneway void)frameChanged: (int)selectedFrame;
-(oneway void)programIsRunning;
-(oneway void)programStopped;
-(oneway void)programIsDead;
-(oneway void)setController: (id <GdbController>)anObject;
-(oneway void)programIsLoaded;
-(BOOL)isNewUser;
//-(oneway void)breakpointAdded: (id <Breakpoint>)breakpoint;
//-(oneway void)breakpointDeleted: (id <Breakpoint>)breakpoint;
//-(oneway void)breakpointChanged: (id <Breakpoint>)breakpoint;
@end

#define ISPOINTER   (1 << 0)
#define ISENUM      (1 << 1)
#define ISSCALER    (1 << 2)
#define ISFLOAT     (1 << 3)
#define ISDOUBLE    (1 << 4)
#define ISAGGREGATE (1 << 5)
#define ISCHAR	    (1 << 6)
#define ISARRAY     (1 << 7)
#define ISSIGNED    (1 << 8)
#define ISOBJECT    (1 << 9)
#define ISSTRING    (1 << 10)
#define ISFUNC      (1 << 11)

