#import "gdb.h"
#import <libc.h>			/* For getenv() */

#import <stdio.h>
#import <mach/msg_type.h>

#import <sys/types.h>
#import <sys/stat.h>
#import <mach/message.h>
#import <mach/cthreads.h>
#import "GdbController.h"
#import <appkit/Listener.h>
#import <appkit/Speaker.h>

#import <objc/error.h>
#import <objc/List.h>

static port_t editorPort = PORT_NULL;
static id gdbSpeaker = nil;
static char *view_program = NULL;
static char *view_host = NULL;
static char gdb_host[128];

extern GdbController *controller, *remote_controller;
extern cthread_t gdb_thread;

void
display_lines (struct symtab *symtab, int first_line, int last_line)
{
  int ok = NO;
  char posString[20];
  char *filename;

  if (gdbSpeaker == nil)		/* make sure we have a speaker */
    return;

  if (editorPort == PORT_NULL)		/* make sure we have a port */
    return;

  filename = symtab->fullname;		/* rooted path */

  if (!filename)
    return;

NX_DURING
    if (first_line == last_line)
	[(cthread_self() == gdb_thread) ? remote_controller : controller 
	    openFile: filename atTrueLine: first_line];
    else
	[(cthread_self() == gdb_thread) ? remote_controller : controller
		openFile: filename fromTrueLine: first_line to: last_line];
NX_HANDLER
  printf("There is some problem with contacting Edit, exception: %d\n",
         NXLocalHandler.code);
NX_ENDHANDLER    

  return;
}


int do_display_lines = 0;

static void
view_command (arg)
     char *arg;
{
  port_t replyPort;
  char *host = arg ? arg : *view_host ? view_host : NULL;
  
  if (gdbSpeaker == nil)		/* make sure we have a speaker */
    gdbSpeaker = [Speaker new];

  editorPort = NXPortFromName (view_program, host);

  if (editorPort == PORT_NULL)
    {
      do_display_lines = 0;
      if (host)
        error ("Can't connect to %s on host %s.", view_program, host);
      else
        error ("Can't connect to %s.", view_program);
    }
  [gdbSpeaker setSendPort: editorPort];
  port_allocate (task_self (), &replyPort);
  [gdbSpeaker setReplyPort: replyPort];
  
  if (controller)
    [(cthread_self() == gdb_thread)
     ? remote_controller : controller disconnect: YES];
  else
    [GdbController new];
  [controller connectToSpeaker: gdbSpeaker];
  
  do_display_lines = 1;
  if (inferior_task)
    tell_client_program_stopped();
  else
    tell_client_program_is_dead();
    
  if (target_has_stack)
    print_sel_frame(1);
  return;
}

static void
unview_command (arg, from_tty)
     char *arg;
     int from_tty;
{
  do_display_lines = 0;
  if (controller)
    [(cthread_self() == gdb_thread)
     ? remote_controller : controller disconnect: YES];
  if (target_has_stack)
    print_sel_frame(1);
  return;
}

void
turn_off_viewing()
{
    do_display_lines = 0;
    if (view_host)
	printf("%s on host %s has gone away.  Viewing is now off.\n",
	       view_program, view_host);
    else
	printf("%s has gone away.  Viewing is now off.\n", view_program);
}

void
set_view_program(char *name)
{
  view_program = savestring(name, strlen(name) + 1);
}

void
set_view_host(char *name)
{
  view_host = savestring(name, strlen(name) + 1);
}

void
_initialize_display ()
{
  add_com ("view", class_files, view_command,
	   "Cause source files to be viewed in Edit.");
  add_com_alias ("v", "view", class_files, 1);
  add_com ("unview", class_files, unview_command,
	   "Cause source files not to be viewed in Edit.");
  if (!view_program)
    view_program = getenv("ViewProgram");
  if (!view_program)
    view_program = NXGetDefaultValue("gdb", "ViewProgram");
  if (!view_program)
    view_program = savestring("Edit", sizeof("Edit"));
  if (!view_host)
    view_host = savestring("", sizeof(""));
  add_show_from_set(
    add_set_cmd("view-program", class_files, var_string_noescape, 
                (char *)&view_program,
                "Set name of program to connect to when viewing.",
		&setlist),
    &showlist);		
  add_show_from_set(
    add_set_cmd("view-host", class_files, var_string_noescape,
                (char *)&view_host,
                "Set host to connect to when viewing.",
		&setlist),
    &showlist);
  gethostname(gdb_host, sizeof(gdb_host));
}
